﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SQLite;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using WSCF;

namespace RemoteSQLite
{
    public class SQLiteRemote : ISQLiteRemoteService
    {
        NLog.Logger logger = NLog.LogManager.GetCurrentClassLogger();

        public event PropertyChangedEventHandler PropertyChanged;

        public SQLiteRemote()
        {

        }
        /// <summary> 
        /// 查询数据库中的所有数据类型信息。
        /// </summary> 
        public void ExecuteReader(string connectionString, string commandText, int millisecondsTimeout, AsyncCBHandler asyncDelegate, object asyncContext)
        {
            Task<DataTable> task = new Task<DataTable>(() =>
            {
                DataTable data;
                using (SQLiteConnection connection = new SQLiteConnection(connectionString))
                {
                    using (SQLiteCommand command = new SQLiteCommand(connection))
                    {
                        connection.Open();
                        command.CommandText = commandText;
                        // 开始读取
                        SQLiteDataAdapter adapter = new SQLiteDataAdapter(command);
                        data = new DataTable();
                        adapter.Fill(data);
                        connection.Close();
                    }
                }
                return data;
            });
            task.Start();
            task.Wait(millisecondsTimeout);

            if (task.IsFaulted)
            {
                logger.Error($"ExecuteReader{connectionString}, {commandText} task.IsFaulted==true");
                asyncDelegate?.Invoke(asyncContext, new ExecuteReaderReponse()
                {
                    ErrMsg = task.Exception.ToString(),
                });
            }
            if (task.IsCompleted)
            {
                asyncDelegate?.Invoke(asyncContext, new ExecuteReaderReponse()
                {
                    Ret = task.Result
                });
            }
            else
            {
                Exception e= new TimeoutException();
                logger.Error(e, $"ExecuteReader {connectionString}, {commandText}");
                asyncDelegate?.Invoke(asyncContext, new ExecuteReaderReponse()
                {
                    ErrMsg = e.ToString(),
                });
                throw e;
            }
        }

        public void ExecuteNonQuery(string connectionString, string commandText, int millisecondsTimeout, AsyncCBHandler asyncDelegate, object asyncContext)
        {
            Task<int> task = new Task<int>(() =>
            {
                int ret = 0;
                using (SQLiteConnection connection = new SQLiteConnection(connectionString))
                {
                    using (SQLiteCommand command = new SQLiteCommand(connection))
                    {
                        connection.Open();
                        command.CommandText = commandText;
                        ret = command.ExecuteNonQuery();
                        connection.Close();
                    }
                }
                return ret;
            });
            task.Start();
            task.Wait(millisecondsTimeout);

            if (task.IsFaulted)
            {
                logger.Error($"ExecuteNonQuery {connectionString}, {commandText} ttask.IsFaulted==true");
                asyncDelegate?.Invoke(asyncContext, new ExecuteNonQueryReponse()
                {
                    ErrMsg = task.Exception.ToString(),
                });
            }
            if (task.IsCompleted)
            {
                asyncDelegate?.Invoke(asyncContext, new ExecuteNonQueryReponse()
                {
                    Ret = task.Result
                });
            }
            else
            {
                Exception e = new TimeoutException();
                logger.Error(e,$"ExecuteNonQuery {connectionString}, {commandText}");
                asyncDelegate?.Invoke(asyncContext, new ExecuteNonQueryReponse()
                {
                    ErrMsg = e.Message
                });
            }
        }

        public void ExecuteScalar(string connectionString, string commandText, int millisecondsTimeout, AsyncCBHandler asyncDelegate, object asyncContext)
        {
            Task<object> task = new Task<object>(() =>
            {
                object ret;
                using (SQLiteConnection connection = new SQLiteConnection(connectionString))
                {
                    using (SQLiteCommand command = new SQLiteCommand(connection))
                    {
                        connection.Open();
                        command.CommandText = commandText;
                        ret = command.ExecuteScalar();
                        connection.Close();
                    }
                }
                return ret;
            });
            task.Start();
            task.Wait(millisecondsTimeout);

            if (task.IsFaulted)
            {
                logger.Error(task.Exception, $"ExecuteScalar {connectionString}, {commandText} task.IsFaulted=true");
                asyncDelegate?.Invoke(asyncContext, new ExecuteScalarReponse()
                {
                    ErrMsg = task.Exception.ToString(),
                });
            }
            if (task.IsCompleted)
            {
                asyncDelegate?.Invoke(asyncContext, new ExecuteScalarReponse()
                {
                    Ret = task.Result
                });
            }
            else
            {
                Exception e = new TimeoutException();
                logger.Error(e, $"ExecuteScalar {connectionString}, {commandText}");
                asyncDelegate?.Invoke(asyncContext, new ExecuteScalarReponse()
                {
                    ErrMsg = e.Message
                });
            }
        }
    }
}
