﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using WSCF;

namespace RemoteSQLite
{
    public class SysTime : ISysTimeService
    {
        public DateTime Now { get; private set; }

        public event PropertyChangedEventHandler PropertyChanged;

        System.Timers.Timer timer;

        public SysTime()
        {
            timer = new System.Timers.Timer();
            timer.AutoReset = true;
            timer.Interval = 1000;
            timer.Elapsed += (s, e) =>
            {
                Now = DateTime.Now;
            };
            timer.Start();
        }

        public void GetTime(AsyncCBHandler asyncDelegate, object asyncContext)
        {
            asyncDelegate?.Invoke(asyncContext, new GetTimeReponse() { now = DateTime.Now });
        }

        
        public void SetTime(DateTime setTime, DateTime reponseTime, AsyncCBHandler asyncDelegate, object asyncContext)
        {
            DateTime now = DateTime.Now;
            var ts = now - reponseTime;
            
            if (ts < TimeSpan.FromSeconds(1) && ts>TimeSpan.Zero)//滞后只能1秒内 
            {
                setTime += TimeSpan.FromTicks(ts.Ticks / 2);
            }
            //转换System.DateTime到SYSTEMTIME

            SYSTEMTIME st = new SYSTEMTIME();

            st.FromDateTime(setTime);


            //调用Win32 API设置系统时间

            Win32API.SetLocalTime(ref st);
        }
    }

    public struct SYSTEMTIME

    {

        public ushort wYear;

        public ushort wMonth;

        public ushort wDayOfWeek;

        public ushort wDay;

        public ushort wHour;

        public ushort wMinute;

        public ushort wSecond;

        public ushort wMilliseconds;



        /// <summary>

        /// 从System.DateTime转换。

        /// </summary>

        /// <param name="time">System.DateTime类型的时间。</param>

        public void FromDateTime(DateTime time)

        {

            wYear = (ushort)time.Year;

            wMonth = (ushort)time.Month;

            wDayOfWeek = (ushort)time.DayOfWeek;

            wDay = (ushort)time.Day;

            wHour = (ushort)time.Hour;

            wMinute = (ushort)time.Minute;

            wSecond = (ushort)time.Second;

            wMilliseconds = (ushort)time.Millisecond;

        }

        /// <summary>

        /// 转换为System.DateTime类型。

        /// </summary>

        /// <returns></returns>

        public DateTime ToDateTime()

        {

            return new DateTime(wYear, wMonth, wDay, wHour, wMinute, wSecond, wMilliseconds);

        }

        /// <summary>

        /// 静态方法。转换为System.DateTime类型。

        /// </summary>

        /// <param name="time">SYSTEMTIME类型的时间。</param>

        /// <returns></returns>

        public static DateTime ToDateTime(SYSTEMTIME time)

        {

            return time.ToDateTime();

        }

    }

    public class Win32API

    {

        [DllImport("Kernel32.dll")]

        public static extern bool SetLocalTime(ref SYSTEMTIME Time);

        [DllImport("Kernel32.dll")]

        public static extern void GetLocalTime(ref SYSTEMTIME Time);

    }
}
