﻿using GalaSoft.MvvmLight.Command;
using RemoteSQLite;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using WSCF;

namespace SQLiteRemoteClientUI
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        MainVm viewModel;
        public MainWindow()
        {
            InitializeComponent();
            viewModel = new MainVm();
            viewModel.Init();
            this.DataContext = viewModel;
        }
    }
    public class MainVm : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        ISQLiteRemoteService sqliteRemoteService;
        ISysTimeService sysTimeService;

        public string Addr { get; set; } = "127.0.0.1:20000";
        public string CommandText { get; set; } = "select * from ScanData";

        public string DbPath { get; set; } = @"D:\flydata\history.sqlite3";
        public string ConnectString => $"Data Source={DbPath};Version=3;";


        public DataTable ReponseTable { get; private set; }
        public bool IsConnected { get; private set; }
        public DateTime Time { get; private set; }
        public string DelayMsg { get; private set; }

        public bool IsConnectEnable { get; private set; } = true;
        #region Command
        public RelayCommand ConnectCmd { get; }
        public RelayCommand ExecuteReaderCmd { get; }
        public RelayCommand GetDelayCmd { get; }
        #endregion

        public MainVm() 
        {
            ConnectCmd = new RelayCommand(Connect);
            ExecuteReaderCmd = new RelayCommand(ExecuteReader);
            GetDelayCmd = new RelayCommand(GetDelay);
        }


        public void Init() 
        {
        
        }

        private void Connect()
        {
            sqliteRemoteService = new SQLiteRemoteServiceClient("SQLiteRemote", false, Addr);
            sysTimeService = new SysTimeServiceClient("SysTime", false, Addr);

            sysTimeService.PropertyChanged += SysTimeService_PropertyChanged;

            IsConnected = ((Reflect_SeviceClient)sysTimeService).IsConnected;
            IsConnectEnable = false;
        }

        private void SysTimeService_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(sysTimeService.Now))
            {
                Time = sysTimeService.Now;
            }
            else if (e.PropertyName == nameof(Reflect_SeviceClient.IsConnected)) {
                IsConnected = ((Reflect_SeviceClient)sysTimeService).IsConnected;
            }
        }

        private void ExecuteReader()
        {
            sqliteRemoteService.ExecuteReader(ConnectString, CommandText, 1000, (asyncContext, retData) =>
            {
                var reponse = retData as ExecuteReaderReponse;
                if (!string.IsNullOrEmpty(reponse.ErrMsg))
                {
                    MessageBox.Show($"出错: {reponse.ErrMsg}");
                }
                else {
                    ReponseTable = reponse.Ret;
                }
            }, this);
        }

        private void GetDelay()
        {
            DateTime time = DateTime.Now;
            sysTimeService.GetTime((asyncContext, retData) =>
            {
                DateTime now = DateTime.Now;
                var reponse = (GetTimeReponse)retData;
                var _time = (DateTime)asyncContext;
                TimeSpan ts = TimeSpan.FromTicks((now - _time).Ticks);
                DelayMsg = $"服务器时间=[{reponse.now}] 通信延时={ts.TotalMilliseconds}ms";
            }, time);
        }

    }
}
