﻿using NLog.LogReceiverService;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;
using WebSocketSharp.Server;

namespace SQLiteRemoteServerUI
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        FLY.AppHelper.WindowNotifyIconHelper notifyiconhelper;

        RemoteSQLite.RemoteSQLiteServer server;
        MainVm viewModel;

        public MainWindow()
        {
            InitializeComponent();


            Assembly asm = Assembly.GetAssembly(typeof(MainWindow));
            string version = "v" + asm.GetName().Version.ToString();
            
            this.Title = "SQLite远程代理" + " " + version;
            notifyiconhelper = new FLY.AppHelper.WindowNotifyIconHelper(this, this.Title);

            this.Hide();

            server = new RemoteSQLite.RemoteSQLiteServer();
            server.Start();

            viewModel = new MainVm();
            viewModel.Init(server.wssv);
            this.DataContext = viewModel;
        }


    }
    public class MainVm
    {
        DispatcherTimer timer;
        
        public string Url { get; set; }
        public ObservableCollection<string> Paths { get; }= new ObservableCollection<string>();
        public ObservableCollection<string> SessionInfos { get; } = new ObservableCollection<string>();

        WebSocketServer wss;
        public void Init(WebSocketServer wss) 
        {
            this.wss = wss;

            timer = new DispatcherTimer();
            timer.Interval = TimeSpan.FromSeconds(1);
            timer.Tick += Timer_Tick;
            timer.Start();

            Url = $"ws://{this.wss.Address}:{this.wss.Port}";
            foreach (var host in this.wss.WebSocketServices.Hosts) {
                Paths.Add(host.Path);
            }
        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            if (wss.WebSocketServices.Hosts.Count() > 0)
            {
                wss.WebSocketServices.Hosts.Select(h => h.Sessions.Sessions.Select(s => s.StartTime));
                List<string> msgs = new List<string>();
                foreach (var host in wss.WebSocketServices.Hosts) {
                    msgs.AddRange(host.Sessions.Sessions.Select(s => $"{host.Path} [{s.ID}] {s.StartTime}"));
                }
                
                for (int i = 0; i < msgs.Count(); i++)
                {
                    var msg = msgs[i];
                    if (i < SessionInfos.Count)
                    {
                        SessionInfos[i] = msg;
                    }
                    else
                    {
                        SessionInfos.Add(msg);
                    }
                }
                while (SessionInfos.Count() > msgs.Count())
                    SessionInfos.RemoveAt(SessionInfos.Count() - 1);
            }
        }
    }
}
