﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using WebSocketSharp;
using WebSocketSharp.Server;


namespace WSCF
{
    public class Reflect_Proxy : WebSocketBehavior,IDisposable
    {
        static NLog.Logger logger = NLog.LogManager.GetCurrentClassLogger();
        //对象的接口类型
        Type interfaceType;
        //代理对象
        object obj;
        //忽略设置
        public bool ignoreSet = false;
        
        List<AnyEvent> anyEvents = new List<AnyEvent>();
        Dictionary<object, string> subProperties = new Dictionary<object, string>();
        List<string> properties = new List<string>();


        public bool isAlive = false;

        
        public Reflect_Proxy()
        {

        }
        public void Init(Type interfaceType, object obj)
        {
            this.interfaceType = interfaceType;
            this.obj = obj;

            InitPropertyChanged();

            //处理[PropertyPush]
            InitPropertyPush();

            //处理[Push]
            InitEventPush();
        }
        List<DisposeAction> DisposeActions = new List<DisposeAction>();
        class DisposeAction 
        {
            public Action<object> Dispose;
            public object Obj;
            public DisposeAction(Action<object> dispose, object obj) 
            {
                this.Dispose = dispose;
                this.Obj = obj;
            }
            public void Do() {
                Dispose(Obj);
            }
        }
        #region Init
        void InitPropertyChanged()
        {
            if (!typeof(INotifyPropertyChanged).IsAssignableFrom(interfaceType))
                return;

            //继承了INotifyPropertyChanged
            ((INotifyPropertyChanged)(this.obj)).PropertyChanged += Obj_PropertyChanged;

            //资源释放时断开引用
            DisposeActions.Add(new DisposeAction((_obj) =>
            {
                ((INotifyPropertyChanged)(_obj)).PropertyChanged -= Obj_PropertyChanged;
            }, this.obj));


            var interfaceTypes = new List<Type>();
            interfaceTypes.Add(this.interfaceType);
            interfaceTypes.AddRange(this.interfaceType.GetInterfaces());

            var propertyInfos = new List<PropertyInfo>();
            foreach (var ifaceType in interfaceTypes)
            {
                propertyInfos.AddRange(ifaceType.GetProperties());
            }

            foreach (var propertyInfo in propertyInfos)
            {

                if (propertyInfo.GetCustomAttribute<JsonIgnoreAttribute>() != null)
                    continue;//这个不需要推送

                if (properties.Contains(propertyInfo.Name))
                    continue;
                properties.Add(propertyInfo.Name);
            }
        }
        void InitEventPush()
        {
            var interfaceTypes = new List<Type>();

            interfaceTypes.Add(this.interfaceType);
            interfaceTypes.AddRange(this.interfaceType.GetInterfaces());


            var eventInfos = new List<EventInfo>();
            foreach (var ifaceType in interfaceTypes)
            {
                eventInfos.AddRange(ifaceType.GetEvents());
            }

            foreach (var eventInfo in eventInfos)
            {
                var pushAttribute = eventInfo.GetCustomAttribute<PushAttribute>();
                if (pushAttribute == null)
                    continue;

                if (anyEvents.Any(ae => ae.eventName == eventInfo.Name))
                    continue;//已经添加了

                var anyEvent = new AnyEvent()
                {
                    eventName = eventInfo.Name,
                    PushObjInfoEx = (rd) => {
                        //数据推送！！！
                        //再嵌套

                        Reflect_OBJ_INTERFACE.PkgData pkgData = new Reflect_OBJ_INTERFACE.PkgData(
                            Reflect_OBJ_INTERFACE.PkgName.PUSH_Event,
                            null,
                            JObject.FromObject(rd)
                        );
                        string json = JsonConvert.SerializeObject(pkgData);
                        SendEx(json);
                    }
                };
                //这个事件需要推送
                eventInfo.AddEventHandler(obj, anyEvent.eventHandler);
                anyEvents.Add(anyEvent);

            }
        }
        void InitPropertyPush()
        {
            //处理[PropertyPush]
            var interfaceTypes = new List<Type>();

            interfaceTypes.Add(this.interfaceType);
            interfaceTypes.AddRange(this.interfaceType.GetInterfaces());

            var propertyInfos = new List<PropertyInfo>();
            foreach (var ifaceType in interfaceTypes)
                propertyInfos.AddRange(ifaceType.GetProperties());


            foreach (var propertyInfo in propertyInfos)
            {
                if (!IsPropertyPush(propertyInfo))
                    continue;

                if (!properties.Contains(propertyInfo.Name))
                {
                    properties.Add(propertyInfo.Name);
                }

                InitSubPropertyPush(this.obj.GetType().GetProperty(propertyInfo.Name), this.obj, null);
            }
        }
        bool IsPropertyPush(PropertyInfo propertyInfo)
        {
            if (propertyInfo.GetCustomAttribute<PropertyPushAttribute>() == null)
                return false;//必须是[PropertyPush]

            if (propertyInfo.CanWrite)
                return false;//它只能是get,不能有set

            if (!typeof(INotifyPropertyChanged).IsAssignableFrom(propertyInfo.PropertyType))
                return false;//必须是从INotifyPropertyChanged派生的

            return true;
        }
        void InitSubPropertyPush(PropertyInfo propertyInfo, object obj, string parentName)
        {
            //下级推送！！！！
            var propertyValue = propertyInfo.GetValue(obj);
            string path = parentName == null ? propertyInfo.Name : $"{parentName}.{propertyInfo.Name}";
            subProperties.Add(propertyValue, path);

            ((INotifyPropertyChanged)(propertyValue)).PropertyChanged += Sub_PropertyChanged;
            
            //资源释放时断开引用
            DisposeActions.Add(new DisposeAction((_obj) =>
            {
                ((INotifyPropertyChanged)(_obj)).PropertyChanged -= Sub_PropertyChanged;
            }, propertyValue));

            //继续向下找
            var subPropertyInfos = propertyInfo.PropertyType.GetProperties();
            foreach (var subPropertyInfo in subPropertyInfos)
            {
                if (!IsPropertyPush(subPropertyInfo))
                    continue;//必须是[PropertyPush]

                InitSubPropertyPush(subPropertyInfo, propertyValue, path);
            }
        }
        private void Obj_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (ignoreSet)//从服务器接收的数据，不用再推送给服务器
                return;

            if (interfaceType == null)
                return;

            if (!properties.Contains(e.PropertyName))
                return;//这个不需要推送

            var v = this.obj.GetType().GetProperty(e.PropertyName).GetValue(sender);
            var jObject = new JObject();
            if (v == null)
                jObject.Add(e.PropertyName, null);
            else
                jObject.Add(e.PropertyName, JToken.FromObject(v));

            Reflect_OBJ_INTERFACE.PkgData pkgData = new Reflect_OBJ_INTERFACE.PkgData(
                Reflect_OBJ_INTERFACE.PkgName.PUSH_PropertyChanged,
                null,
                jObject
            );
            string json = JsonConvert.SerializeObject(pkgData);
            SendEx(json);
        }
        void SendEx(string msg) {
            try
            {
                Send(msg);
            }
            catch (Exception e)
            {
                //异常,通常是断开了！！
                logger.Error(e);
            }
        }
        private void Sub_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (ignoreSet)//从服务器接收的数据，不用再推送给服务器
                return;
            if (!subProperties.ContainsKey(sender))
                return;//异常, 不是子property

            var type = sender.GetType();

            var propertyInfo = type.GetProperty(e.PropertyName);
            if (propertyInfo == null)
                return;//获取失败！！！

            if (propertyInfo.GetCustomAttribute<JsonIgnoreAttribute>() != null)
                return;//这个不需要推送

            string path = subProperties[sender];
            string[] parentNames = path.Split('.');
            if (parentNames.Count() == 0)
                return;//分解出错

            var v = propertyInfo.GetValue(sender);
            var jObject = new JObject();
            if (v == null)
                jObject.Add(propertyInfo.Name, null);
            else
                jObject.Add(propertyInfo.Name, JToken.FromObject(v));

            JObject jObject_parent = null;
            for (int i = 0; i < parentNames.Count(); i++)
            {
                int idx = parentNames.Count() - 1 - i;
                jObject_parent = new JObject();
                jObject_parent.Add(parentNames[idx], jObject);
                jObject = jObject_parent;
            }

            Reflect_OBJ_INTERFACE.PkgData pkgData = new Reflect_OBJ_INTERFACE.PkgData(
                Reflect_OBJ_INTERFACE.PkgName.PUSH_PropertyChanged,
                null,
                jObject_parent
            );
            string json = JsonConvert.SerializeObject(pkgData);
            SendEx(json);
        }
        #endregion


        protected override void OnOpen()
        {
            base.OnOpen();
            isAlive = true;
        }

        protected override void OnClose(CloseEventArgs e)
        {
            base.OnClose(e);
            isAlive = false;

            Dispose();
        }

        protected override void OnMessage(MessageEventArgs e)
        {
            if (!e.IsText)
                return;
            string json = e.Data;
            var p = JsonConvert.DeserializeObject<Reflect_OBJ_INTERFACE.PkgData>(json);
            OnMessage(p);
        }

        void OnMessage(Reflect_OBJ_INTERFACE.PkgData pkgData)
        {
            var pkgName = Enum.Parse(typeof(Reflect_OBJ_INTERFACE.PkgName), pkgData.name);
            switch (pkgName)
            {
                case Reflect_OBJ_INTERFACE.PkgName.CALL_GetAllProperties:
                    {

                        var jObject = new JObject();

                        var type = obj.GetType();

                        foreach (var propertyName in properties)
                        {
                            var propertyInfo = type.GetProperty(propertyName);
                            var v = propertyInfo.GetValue(obj);
                            if (v == null)
                                jObject.Add(propertyInfo.Name, null);
                            else
                                jObject.Add(propertyInfo.Name, JToken.FromObject(v));
                        }
                        pkgData.data = jObject;

                        string json = JsonConvert.SerializeObject(pkgData);
                        SendEx(json);
                    }
                    break;
                case Reflect_OBJ_INTERFACE.PkgName.CALL_SetProperty:
                    {
                        ignoreSet = true;
                        //TODO
                        string json = pkgData.data.ToString();
                        JsonConvert.PopulateObject(json, obj);

                        ignoreSet = false;
                    }
                    break;
                case Reflect_OBJ_INTERFACE.PkgName.CALL_MethodInvoke:
                    {
                        var rData = pkgData.data.ToObject<Reflect_OBJ_INTERFACE.ReflectData>();
                        var type = obj.GetType();
                        var methodInfo = type.GetMethod(rData.name);
                        var parameterInfos = methodInfo.GetParameters();
                        object[] parameters = new object[parameterInfos.Count()];
                        for (int i = 0; i < parameters.Count(); i++)
                        {
                            var ptype = parameterInfos[i].ParameterType;
                            var pname = parameterInfos[i].Name;
                            if (string.Compare(pname, Reflect_OBJ_INTERFACE.asyncDelegate, true) == 0)
                            {
                                parameters[i] = new AsyncCBHandler(asyncDelegate);
                            }
                            else if (string.Compare(pname, Reflect_OBJ_INTERFACE.asyncContext, true) == 0)
                            {
                                parameters[i] = new CC() { methodName = rData.name, guid = pkgData.guid };
                            }
                            else
                            {
                                parameters[i] = rData.data[pname].ToObject(ptype);
                            }
                        }
                        methodInfo.Invoke(obj, parameters);
                    }break;
             }
            

        }

        void asyncDelegate(object asyncContext, object retData)
        {
            var cc = (CC)asyncContext;

            var rData = new Reflect_OBJ_INTERFACE.ReflectData()
            {
                name = cc.methodName,
                data = retData == null ? null : JToken.FromObject(retData)
            };


            Reflect_OBJ_INTERFACE.PkgData pkgData = new Reflect_OBJ_INTERFACE.PkgData(
                Reflect_OBJ_INTERFACE.PkgName.CALL_MethodInvoke,
                cc.guid,
                JToken.FromObject(rData)
            );

            string json = JsonConvert.SerializeObject(pkgData);
            SendEx(json);
        }

        public void Dispose()
        {
            foreach (var act in DisposeActions) {
                act.Do();
            }
            DisposeActions.Clear();
        }

        class CC
        {
            public string methodName;
            public string guid;
        }
    
        
    }
    /// <summary>
    /// 事件推送包装
    /// </summary>
    class AnyEvent
    {
        /// <summary>
        /// 事件名称
        /// </summary>
        public string eventName;
        /// <summary>
        /// json包装后的ReflectData 推送
        /// </summary>
        public Action<Reflect_OBJ_INTERFACE.ReflectData> PushObjInfoEx;
        /// <summary>
        /// 给 EventInfo 调用
        /// </summary>
        public EventHandler eventHandler;
        public AnyEvent()
        {
            eventHandler = new EventHandler(Obj_AnyEvent);
        }
        void Obj_AnyEvent(object sender, EventArgs e)
        {
            var rData = new Reflect_OBJ_INTERFACE.ReflectData()
            {
                name = eventName,
                data = JObject.FromObject(e)
            };
            PushObjInfoEx?.Invoke(rData);
        }
    }
}
