﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using WebSocketSharp;
using WebSocketSharp.Server;

namespace WSCF
{
    public class WSProxy : WebSocketBehavior
    {
        object sender;
        public bool isAlive = false;

        protected Dictionary<string, Action<WSJsonPackage>> Funcs = new Dictionary<string, Action<WSJsonPackage>>();

        public WSProxy()
        {
            Funcs.Add("GetAllValue", GetAllValue);
            Funcs.Add("SetValue", SetValue);

        }
        public virtual void Init(object sender)
        {
            this.sender = sender;
            if (sender is INotifyPropertyChanged)
            {
                (sender as INotifyPropertyChanged).PropertyChanged += sender_PropertyChanged;
            }
        }

        private void sender_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (!isAlive)
                return;
            Type type = sender.GetType();
            try
            {
                var obj = type.GetProperty(e.PropertyName).GetValue(sender);


                WSJsonPackage wSJsonPackage = new WSJsonPackage()
                {
                    Guid = Guid.NewGuid().ToString(),
                    Func = "PropertyChanged",
                    Data = JsonConvert.SerializeObject(
                        new WSJsonPropertyValue()
                        {
                            PropertyName = e.PropertyName,
                            PropertyValue = JsonConvert.SerializeObject(obj)
                        })
                };

                Send(JsonConvert.SerializeObject(wSJsonPackage));
            }
            catch
            {
                //error
            }
        }

        void GetAllValue(WSJsonPackage request)
        {

            string guid = request.Guid;

            Type type = sender.GetType();
            var properties = type.GetProperties();
            WSJsonGetAllPropertyValue p = new WSJsonGetAllPropertyValue();
            p.PropertyValues = new List<WSJsonPropertyValue>();
            foreach (var property in properties)
            {
                var obj = property.GetValue(sender);
                p.PropertyValues.Add(new WSJsonPropertyValue()
                {
                    PropertyName = property.Name,
                    PropertyValue = JsonConvert.SerializeObject(obj)
                });
            }

            WSJsonPackage reponse = new WSJsonPackage()
            {
                Guid = guid,
                Func = "GetAllValue",
                Data = JsonConvert.SerializeObject(p)
            };

            Send(JsonConvert.SerializeObject(reponse));
        }

        void SetValue(WSJsonPackage request)
        {
            var request_obj = JsonConvert.DeserializeObject<WSJsonPropertyValue>(request.Data);

            string propertyname = request_obj.PropertyName;
            string propertyvalue = request_obj.PropertyValue;

            Type type = sender.GetType();
            try
            {
                var property = type.GetProperty(propertyname);

                var value = JsonConvert.DeserializeObject(propertyvalue, property.PropertyType);

                property.SetValue(sender, value);
            }
            catch
            {
                //异常
            }
        }
        protected override void OnOpen()
        {
            base.OnOpen();
            isAlive = true;
        }

        protected override void OnClose(CloseEventArgs e)
        {
            base.OnClose(e);
            isAlive = false;
        }

        protected override void OnMessage(MessageEventArgs e)
        {
            if (!e.IsText)
                return;
            string json = e.Data;
            var p = JsonConvert.DeserializeObject<WSJsonPackage>(json);
            OnMessage(p);
        }

        bool OnMessage(WSJsonPackage wSJsonPackage)
        {
            string func = wSJsonPackage.Func;
            if (Funcs.Keys.Contains(func))
            {
                Funcs[func](wSJsonPackage);
                return true;
            }
            return false;
        }
    }
}
